#!/usr/bin/perl -w
# $Id: forecast.pl,v 1.6 2020/12/19 18:33:13 teus Exp teus $

# Copyright (C) 2016, Teus Hagen, the Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# script will download forecasting info for a location (see %locations) from
#       weather www.yr.no (two days forecast): temp, cloud, pressure, rain
#       pollutant PM2.5 www.aqicn.org (4 days forecast): AQI convert to PM2.5
# generate json for weather and pollutant Index values (LKI/AQI)
# edit the HTML template file to generate website ready HTML script to output file

use JSON;
use Time::Piece;
use POSIX qw(strftime);
use feature "state";

my $debug = 0;   # on debug output full HTML page to output
my $verbose = 0; # verbosity level
my $archive = 0; # archive downloaded forecasts
my $Dflt_template ; #   = './forecast.html';       # default HTML template

# reminder: PM10 -> PM2.5 ~= 814.2 + 0.468*PM10

# TO DO: get location URL parts automatically
my $location_dflt = 'Grubbenvorst';
my %locations = (
    Grubbenvorst => { # default location
        # YR.no website location part in URL
        weather => 'Netherlands/Limburg/Grubbenvorst',          # weather forecast
        # AQICN.org website location part in URL
        PM      => 'netherland/horst-a/d-maas/hoogheide/',      # PM25 forecast
    },
    Horst => {
        weather => 'Netherlands/Limburg/Horst',
        PM      => 'netherland/horst-a/d-maas/hoogheide/',
    },
    'Horst a/d Maas' => {
        weather => 'Netherlands/Limburg/Horst_aan_de_Maas',
        PM      => 'netherland/horst-a/d-maas/hoogheide/',
    },
    Venray => {
        weather => 'Netherlands/Limburg/Venray',
        PM => 'netherland/vredepeel/vredeweg/',
    },
    MaastrichtHF => {
        weather => 'Netherlands/Limburg/Maastricht',
        PM => 'netherland/maastricht/hoge-fronten',
    },
    Maastricht => {
        weather => 'Netherlands/Limburg/Maastricht',
        PM => 'netherland/maastricht/a2-nassaulaan',
    },
    Geleen => {
        weather => 'Netherlands/Limburg/Geleen',
        PM => 'netherland/geleen/asterstraat',
    },
    Heerlen => {
        weather => 'Netherlands/Limburg/Heerlen',
        PM => 'netherland/heerlen/jamboreepad',
    },
    HeerlenL => {
        weather => 'Netherlands/Limburg/Heerlen',
        PM => 'netherland/heerlen/looierstraat',
    },
    Nettetal => {
        weather => 'Germany/North_Rhine-Westphalia/Nettetal/',
        PM => 'germany/nrw/nettetal-kaldenkirchen/',
    },
    Wijnandsrade => {
        weather => 'Netherlands/Limburg/Wijnandsrade/',
        PM => 'netherland/wijnandsrade/opfergelstraat/',
    },
    Amsterdam => {
        weather => 'Netherlands/North_Holland/Amsterdam',
        PM => 'netherland/amsterdam',
    },
);
# needs a collection of images for the different weather symbols
# maybe download this image file on installation
my $symbolsURL =
# 'https://www.highcharts.com/samples/graphics/meteogram-symbols-30px.png';
'http://behouddeparel.nl/sites/behouddeparel.nl/files/luchtmetingen/meteogram-symbols-18px.png';

# cached perl format data. This allows to be updated separately
# may stored/obtained as json format filesystem or DB
# cache{weather|pollutant|flagged}{data,lastupdate,nextupdate}
# time in UNIX timestamp, data in perl data format (no refs)
# TO DO: sign or decrypt the stored cache data.
my %cache;      # cached perl format data
my $cache_file = '/var/tmp/forecasts/cache-XXXX.json';
sub store_cache {
    return 1 if (not defined $cache{flagged}) || (not $cache{flagged});
    $cache{flagged} = 0;
    my $json = JSON->new->allow_nonref;
    my $cast;
    if( $debug ) {
        $cast = $json->pretty->encode(\%cache); # perl hash table of the foprecast
    } else {
        $cast = $json->encode(\%cache); # perl hash table of the foprecast
    }
    my $FILE;
    if( not open $FILE, ">$cache_file" ) {
        print STDERR "Forecast WARNING: cannot write cache file $cache_file.\n";
        return 0;
    }
    print $FILE $cast; close $FILE;
    print STDERR "Forecast Updated and archived the cache to $cache_file.\n"
        if $verbose > 1;
    return 1;
}

# make sure we keep the cached data
END { store_cache(); }
    
# get ref to perl data structure for this weather or pollutant data
sub get_cache {
    my $type = shift;
    return undef if ($type !~ /^(weather|pollutant)$/);
    return undef if (defined $cache{flagged}) && (not defined $cache{$type}{data});
    return \$cache{$type}{data} if (defined $cache{$type}{data}) && (time <= $cache{$type}{nextupdate});
    # have some data already downloaded?
    my $FILE;
    return undef if not open $FILE, "<$cache_file";
    local $/; # in slurp modus
    my $json = JSON->new->allow_nonref; my $strg = <$FILE>;
    %cache = %{$json->decode($strg)};
    close $FILE;
    return undef if (not defined $cache{$type}{data}) || (time > $cache{$type}{nextupdate});
    $cache{flagged} = 0;
    return \$cache{$type}{data};
}

# agents to use to masquerate download actions
my %agents = (
    'dotbot' => "Mozilla/5.0 (compatible; DotBot/1.1; http://www.opensiteexplorer.org/dotbot, help\@moz.com)",
    'butterfly' => "Mozilla/5.0 (Butterfly)",
    'baidu' => "Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html)",
    'slurp' => "Mozilla/5.0 (compatible; Yahoo! Slurp; http://help.yahoo.com/help/us/ysearch/slurp)",
    'google' => "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)",
    'opera' => "Opera/8.01 (Windows NT 5.1; U; nl)",
    'iphone' => "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_2 like Mac OS X) AppleWebKit/537.51.2 (KHTML, like Gecko) Version/7.0 Mobile/11D257 Safari/9537.53",
    'ipad' => "Mozilla/5.0 (iPad; CPU OS 7_1_2 like Mac OS X) AppleWebKit/537.51.2 (KHTML, like Gecko) Version/7.0 Mobile/11D257 Safari/9537.53",
    'baidu' => "Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html)",
#    'spinn' => "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.19; aggregator:Spinn3r (Spinn3r 3.1); http://spinn3r.com/robot) Gecko/2010040121 Firefox/3.0.19",
    'ahrefsbot' => "Mozilla/5.0 (compatible; AhrefsBot/5.0; +http://ahrefs.com/robot/)",
    'msnbot' => "msnbot-media/1.1 (+http://search.msn.com/msnbot.htm)",
    'majestic' => "Mozilla/5.0 (compatible; MJ12bot/v1.4.4; http://www.majestic12.co.uk/bot.php?+)",
    'windows' => "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/20100101 Firefox/18.0",
);

use LWP::UserAgent;
# get content from a website page: argument URL
sub get_file {
    my $file = shift;
    my $URL;
    if( not $URL ){
        $agnt = ((keys %agents)[int rand keys %agents]);
        $agnt = $agents{$agnt};
        $URL=LWP::UserAgent->new(agent => $agnt);
        $URL->timeout('40');
        # use proxy to anonimize downloads and add advertise income to these sites
        if( system("pidof privoxy >/dev/null") ){
            $URL->env_proxy;
        } else {
            $URL->proxy('http', 'http://127.0.0.1:8118/');
        }
    }
    my $resp = $URL->get($file);

    if( $resp->is_success ){
        return $resp->content;
    } else {
        print STDERR "Forecast ERROR Response status line: \n\t";
        print STDERR $resp->status_line,"\n";
        print STDERR "\nForecast ERROR Cannot obtain data from URL $file\n";
    }
    return undef;
}

# convert string with leading interger part to integer
sub integer {
    my $i = shift; my @ar = split /\n/, $i; $i = $ar[0];
    return undef if not defined $i;
    $i =~ s/^[^0-9]*([0-9]+)[^0-9]*$/$1/;
    return $i;
}

# routines to be able to calculate pollutant values and LKI values
require ('/webdata/luchtmetingen/AQI.pl');
my $AQI_Indices = AQI_Index();
# calculate pm2.5 value from AQI binary search
sub Get_PM25_AQI {
    my ($AQI,$min,$max) = @_;
    $min = 0 if not defined $min; $max = 500 if not defined $max;
    my $val = ($max - $min)/2;
    my @aqi = $AQI_Indices->{AQI}{routine}->("noprint pm_25=$val");
    if( $aqi[0]+1.5 > $AQI ){ return Get_PM25_AQI($AQI,$min,$val); }
    elsif( $aqi[0] - 1.5 < $AQI ){ return Get_PM25_AQI($AQI,$val,$max); }
    else { return int($val*10+0.5)/10; }
}
# calculate pm2.5 value from AQI stepwise
sub GET_PM25 {
    my ($AQI,$val,$step) = @_;
    $val = 1 if not defined $val;
    $step = 20 if not defined $step;
    return int($val*10+0.5)/10 if $step < 1;
    for (my @cur = $AQI_Indices->{AQI}{routine}->("noprint pm_25=$val");
        $cur[0] <= $AQI; ){
        $val += $step;
        @cur = $AQI_Indices->{AQI}{routine}->("noprint pm_25=$val");
    }
    $val -= $step;
    return GET_PM25($AQI,$val,$step/2);
}

# test routine
sub Get_PM25_LKI {
    my $AQI = shift;
    my $PM25 = GET_PM25($AQI); # Get_PM25_AQI($AQI);
    my @lki = $AQI_Indices->{LKI}{routine}->("noprint pm_25=$PM25");
    printf("AQI(%d): PM2.5=%2.1f, LKI: %2.1f,'#%6.6X','%s'",
        $AQI, $PM25, $lki[0], $lki[1], $lki[2]);
}
# Get_PM25_LKI(51);

# get forecast pm2.5 AQI values from forecast URL AQICN.org in China
sub get_PM_forecast {
    state %PMforecast; 
    my ($URL, $from, $hours) = @_;
    return $PMforecast{$URL} if defined $PMforecast{$URL};
    $hours = 0 if not defined $hours;
    $from = 0 if not defined $from;
    my $cast;
    my $json = JSON->new->allow_nonref;
    my %rts;

    $cast = get_cache('pollutant');
    if( not defined $cast ) {
        my $data;
        # from World Air Quality Index (WAQI) project (AQICN.org)
        my $forecast = get_file('http://aqicn.org/city/'.$URL);
        return undef if not defined $forecast;

        my @line = split '\n', $forecast;
        # pick up from website the json pollutant forecast data
        for( my $ln = 0; $ln <= $#line; $ln++ ){
            next if $line[$ln] !~  /function showCityWindForecast.idx./;
            $line[$ln] =~ s/.*function showCityWindForecast.idx..\s+var\s+f=//;
            $line[$ln] =~ s/;try\s+.*//;
            $forecast = $line[$ln];           # the json data
            $forecast = $json->decode($forecast); # perl hash table of the foprecast
            last;
        }
        return undef if (not defined $forecast) || ($#{$forecast} < 0);
        if( (not defined $forecast->[0]{d}) || ($#{$forecast->[0]{d}} < 16) ) {
            # [0]{n} is name, [0]{d} is array of hashed entries, one per three hours
            # entries: t(temp),
            # weather info from Citizen Weather Observer Program (CWOP/APRS)
            #          wind: array (3X hash entries):
            #                       at(time), d(rain), h(pressure), ws(speed),
            #                       wg(gauge), wd(direction), rh(huminity), t(temp),
            #          aqi (c(2Xcolor),v(2Xvalue)) 8 days forecast
            # AQI forecast is is based on Copernicus Atmosphere Monitoring Service (CAMS) 
            printf STDERR ("Forecast FAILURE: downloaded %d pollutant values <48 hours data.\n",
                (defined $forecast->[0]{d})? ($#{$forecast->[0]{d}}+1)*3 : 0);
            return undef;
        }
        $forecast->[0]{n} =~ s/,\s([a-z]\s[a-z]+),\s+(.*)$/, $2\/$1/i; # location
        $cache{flagged} = 1;
        $cache{'pollutant'}{data} = $forecast; $cast = $cache{'pollutant'}{data};
        $cache{'pollutant'}{lastupdate} = time;
        $cache{'pollutant'}{nextupdate} = time + 23*60*60;
        # the location identifier
        if( $archive > 0 && $debug ){ # save the collected pollutant PM2.5 forecast
            my $FILE;
            my $JSON = JSON->new->allow_nonref;
            my $name = strftime("PM2.5-Forecast-%Y-%m-%d_%Hh%M.json",localtime(time));
            open $FILE, ">./$name";
            print STDERR "Forecast Created pollutant PM2.5 forecast $forecast->[0]{n} file: ./$name\n"
                if $verbose;
            print $FILE $json->pretty->encode($cast);
            close $FILE;
        }
        store_cache();
    } else { $cast = ${$cast}; }
    $rts{location} = $cast->[0]{n};
    my $strt_time = 0; $last_time = 0;
    $rts{pm25} = []; my $skip = 0;
    # synchronize the date/time values with weather data
    # convert the AQI values back to PM2.5 ug/m3 values
    for( my $i=0; $i <= $#{$cast->[0]{d}}; $i++){ # walk through the data array
        next if not $cast->[0]{d}[$i]{t};
        next if not $cast->[0]{d}[$i]{aqi}{v};
        my $time = Time::Piece->strptime("$cast->[0]{d}[$i]{t} +0100","%Y-%m-%d %H:%M:%S %z")->epoch;
        if( not $strt_time ){ # only once at the start
            $strt_time = $time;
            $last_time = $time - 60*60;
            $from = $time if (not $from); # first weather date/time
            printf STDERR ("Forecast Start times: pollutant data from %s, weather data from %s\n",
                strftime("%m/%d %H:%M",localtime($time)),
                strftime("%m/%d %H:%M",localtime($from)) )
              if $verbose;
            # case: weather start > pollutant start date/time: prepend zero values
            printf STDERR ("Forecast WARNING: add %d PM2.5=0 values %s tot %s no pollutant data!\n",
                int(($time - ($from + 30*60))/3600),
                strftime("%Y/%m/%d %H:%M",localtime($from)),
                strftime("%m/%d %H:%M",localtime($time)))
              if $verbose && ($from+30*60 < $time) ;
            for( my $pt = $from + 30*60; $pt < $time; $pt += 60*60){
                # initial sync with weather hourly values
                push @{$rts{pm25}}, 0;
            }
        }
        # case: pollutant array is missing some hours (should not happen).
        if( $time > $last_time + 60*60 ){
            printf STDERR ("Forecast WARNING: between %s tot %s no AQI data!\n", strftime("%Y/%m/%d %H:%M",localtime($last_time)), strftime("%m/%d %H:%M",localtime($time)));
            while( $time > $last_time+60*60 ){ # fill the gaps
                $last_time += 60*60;
                push @{$rts{pm25}}, 0;
            }
        }
        my @aqi = (0,0,0); 
	if ( (defined $cast->[0]{d}[$i]{aqi}{v}) and (ref($cast->[0]{d}[$i]{aqi}{v}) eq "HASH") and (defined $cast->[0]{d}[$i]{aqi}{v}{pm25}) ) {
            $aqi[0] = $cast->[0]{d}[$i]{aqi}{v}{pm25}[0] if $cast->[0]{d}[$i]{aqi}{v}{pm25}[0];
            $aqi[2] = $cast->[0]{d}[$i]{aqi}{v}{pm25}[1] if $cast->[0]{d}[$i]{aqi}{v}{pm25}[1];
            $aqi[1] = int( ($aqi[0] + $aqi[2])/2 + 0.5)
                if $cast->[0]{d}[$i]{aqi}{v}{pm25}[0] && $cast->[0]{d}[$i]{aqi}{v}{pm25}[1];
	}

        for( my $i = 0; $i <= $#aqi; $i++ ){
            # case pollutant starts earlier as weather data: skip the pol data
            if( $time + 30*60 < $from ){
                $skip++;
            } else {
                push @{$rts{pm25}}, GET_PM25($aqi[$i]) if $aqi[$i];
                push @{$rts{pm25}}, 0 if not $aqi[$i];
            }
            $time += 60*60; $last_time += 60*60;
            # limit the array length to array length of weather
            last if ($last_time >= ($from + $hours*60*60)) && not $hours;
        }
        # limit the array length to array length of weather
        last if ($last_time >= ($from + $hours*60*60)) && not $hours;
    }
    printf STDERR ("Forecast Attention:\n\tskip $skip pol data\n\tfrom %s upto %s start weather data,\n\tdeleted %d values from end!\n",
        strftime("%Y/%m/%d %H:%M",localtime($strt_time)),
        strftime("%m/%d %H:%M",localtime($from)),
        $hours < ($#{$rts{pm25}}+1) ? ($#{$rts{pm25}} + 1 -  $hours): 0)
                if ($verbose > 1) && $skip;
    $rts{start} = $strt_time + $skip*60*60;
    my $nr = $#{$rts{pm25}}+1 + $skip;
    $#{$rts{pm25}} = $hours-1 if $hours < ($#{$rts{pm25}}+1); # delete unused values
    $rts{hours} = $#{$rts{pm25}}+1;
    printf STDERR ("Forecast Pollutant PM2.5 values: from %s (%d hours), corrected start: %s (%d hours).\n",
        strftime("%Y/%m/%d %H:%M",localtime($strt_time)),
        $nr,
        strftime("%Y/%m/%d %H:%M",localtime($rts{start})),
        $#{$rts{pm25}}+1) if $verbose > 1;
    printf STDERR ("Forecast Location: %s\n\tstart: %s\n\tnr hours: %d\n\tAQI/hour: %s\n",
        $rts{location},
        strftime("%Y-%m-%d %H:%M:%S",localtime($rts{start})),
        $rts{hours}, join ', ', @{$rts{pm25}})
      if $debug > 1;
    $PMforecast{$URL} = \%rts;
    return $PMforecast{$URL};
}

# get_PM_forecast($U);

# obtain the json data for PM25 forecast for next 4 days (99 hours)
sub json_PM_forecast {
    my $aqicn = shift;
    $aqicn = $locations{$location_dflt}{PM} if not defined $aqicn;
    $aqicn = $locations{$aqicn}{PM} if defined $locations{$aqicn}{PM};
    my $aqi_type = shift; $aqi_type = 'LKI' if not defined $aqi_type;
    my $from = shift;
    my $hours = shift; $hours = 0 if not defined $hours;
    my $json = JSON->new->allow_nonref;

    my $pm25 = get_PM_forecast($aqicn, $from, $hours);
    return undef if not defined $pm25;
    return $pm25 if (defined $pm25->{$aqi_type}{pm25});
    $pm25->{$aqi_type}{pm25} = [];
    $pm25->{$aqi_type}{colors} = [];
    $pm25->{$aqi_type}{qual} = [];

    # convert PM2.5 to LKI/AQI Index values
    for( my $i = 0; $i <= $#{$pm25->{pm25}}; $i++ ){
        if( (not defined $pm25->{pm25}[$i]) || (not $pm25->{pm25}[$i]) ){
            # $pm25->{$aqi_type}{pm25}[$i] = 0;
            # $pm25->{$aqi_type}{colors}[$i] = '#0f0f0f';
            # $pm25->{$aqi_type}{qual}[$i] = '?';
            $pm25->{$aqi_type}{pm25}[$i] = undef;
            $pm25->{$aqi_type}{colors}[$i] = undef;
            $pm25->{$aqi_type}{qual}[$i] = undef;
            next;
        }
        my @aqi = $AQI_Indices->{$aqi_type}{routine}->("noprint pm_25=$pm25->{pm25}[$i]");
        $pm25->{$aqi_type}{pm25}[$i] = int($aqi[0]*10+0.5)/10;
        $pm25->{$aqi_type}{colors}[$i] = sprintf("#%0.6X",$aqi[1]);
        $pm25->{$aqi_type}{qual}[$i] = $aqi[2];
    }
    printf STDERR ("Forecast Pollutant %s(pm2.5) starts at %s (%d), has %d hourly values.\n",
        $aqi_type,
        strftime("%Y-%m-%d %H:%M:%S",localtime($pm25->{start})),
        $pm25->{start},
        $#{$pm25->{pm25}}+1)
      if $verbose > 1;
    $pm25->{start} = "$pm25->{start}";
    $pm25->{$aqi_type}{pm25} = $json->encode($pm25->{$aqi_type}{pm25});
    printf("var %s_pm25 = %s;\n", $aqi_type, $pm25->{pm25})
        if $debug > 1;
    $pm25->{$aqi_type}{colors} = $json->encode($pm25->{$aqi_type}{colors});
    printf("var %s_pm25_colors = %s;\n", $aqi_type, $pm25->{$aqi_type}{colors})
        if $debug > 1;
    $pm25->{$aqi_type}{qual} = $json->encode($pm25->{$aqi_type}{qual});
    printf("var %s_pm25_qual = %s;\n", $aqi_type, $pm25->{$aqi_type}{qual})
        if $debug > 1;
    return $pm25;
}
# json_PM_forecast('Grubbenvorst','LKI');
my @ToDutch = (
    ['Calm','windstilte'],
    ['Light breeze','lichte wind'],
    ['Gentle breeze','zwakke wind'],
    ['Moderate breeze','matige bries'],
    ['Fresh breeze','frisse bries'],
    ['Strong breeze','bries'],
    ['Near gale','harde wind'],
    ['Gale','krachtige wind'],
    ['Strong gale','stormachtig'],
    ['Storm','storm'],
    ['Violent storm','zware storm'],
    ['Hurricane','orkaan'],
    ['[Ss]ky','lucht'],
    ['Light air','heldere lucht'],
    ['Partly','gedeeltelijk'],
    ['[Ff]air','kalme'],
    ['[Cc]lear','heldere'],
    ['[Gg]entle','zachte'],
    ['[Ll]ight','lichte'],
    ['[Mm]oderate','milde'],
    ['[Bb]reeze','bries'],
    ['[Cc]loudy','bewolkt'],
    ['[Ss]unny','zonnig'],
    ['[Rr]ain','regen'],
    ['[Ss]outh','zuid'],
    ['[Ww]est','west'],
    ['[Nn]orth','noord'],
    ['[Ee]ast','oost'],
    ['[Ff]og','mist'],
    ['[Cc]alm','windstilte'],
    ['[Gg]ale','harde wind'],
    ['[Ss]trong','stormachtig'],
    ['Storm','storm'],
    ['[Vv]iolent','sterke'],
    ['[Hh]uricane','hevige'],
    ['^Grubbenvorst$','Grubbenvorst, Limburg'],
);

sub CheckTranslateTable {
    return 1 if not $verbose;
    my $strg = shift;
    return 0 if not $strg;
    my $pos = -1; my %found;
    while(($pos= index( $strg, '"name":"', $pos+1)) > -1 ){
        my $name = substr($strg, $pos+8, 20);
        $name =~ s/".*//;
        my $fnd = 0;
        for( my $i = 0; $i <= $#ToDutch; $i++ ){
            if( $name =~ /$ToDutch[$i][0]/ ){ $fnd++; last ; }
        }
        $found{$name} = 0 if not $fnd;
    }
    foreach( keys %found ){
        printf STDERR ("Forecast Name \"%s\" not in translation table.\n", $_);
    }
    return 1;
}

# do translation on a perl data hash table structure with names as key
# default hash table weather cache data entry and default key 'name'
sub Translate_names {
    my $table = shift; $table = \$cache{weather}{data} if not defined $table;
    my $key = shift; $key = 'name' if not defined $key;
    my $flagged = 0;
    # get a ref to all hash key entries with the key name
    my @arr = WalkTree($table,$key);
    if( $#arr < 0 ){
        print STDERR "Forecast WARNING: no names to translate found in weather data.\n";
        return 0;
    }
    for( my $j = 0; $j <= $#arr; $j++ ){
        for( my $i=0; $i <= $#ToDutch; $i++){
            if( ${$arr[$j]} =~ /$ToDutch[$i][0]/ ){
                ${$arr[$j]} =~ s/$ToDutch[$i][0]/$ToDutch[$i][1]/ig;
                $flagged++;
            }
        }
    }
    return $flagged;
}

# do translateion in a json string
sub translate {
    my $strg = shift;
    CheckTranslateTable( $strg );
    for( my $i=0; $i <= $#ToDutch; $i++){
        $strg =~ s/$ToDutch[$i][0]/$ToDutch[$i][1]/ig;
    }
    return $strg;
}

# search a struct for a hash key (not an HASH or ARRAY) and return the address
# return a REF to the searched key
sub WalkTree {
    my $table = shift;
    my $search = shift;
    my @array;
    return @array if (not defined $search) || (not defined $table);
    $table = ${$table} if scalar $table =~ /^REF/;
    if( scalar $table =~ /^HASH/ ) {
        foreach my $key (keys %{$table} ){
            if( $key eq $search  ){
        #&& ((scalar $table->{$key}) !~ /^(ARRAY|HASH|REF)/) ){
#    print STDERR "Forecast *** FOUND key $key ***********\n";
                push @array, \$table->{$key};
            } elsif( scalar $table->{$key} =~ /^(HASH|ARRAY|REF)/ ){
#    print STDERR "Forecast walk hash table on key $key searching for $search\n";
                my @new = WalkTree(\$table->{$key}, $search);
                push @array, @new if $#new >= 0;
            }
            else {
#    print STDERR "Forecast leaf node key $key searching for $search\n";
            }
        }
    } elsif( scalar $table =~ /^ARRAY/ )  {
#    printf STDERR ("Forecast walk into array length %d for search $search\n", $#{$table});
        for( my $i = 0; $i <= $#{$table}; $i++ ) {
            if( scalar ${$table}[$i] =~ /^(HASH|ARRAY|REF)/ ) {
                my @new = WalkTree(\${$table}[$i], $search);
                push @array, @new if $#new >= 0;
#    printf STDERR "Forecast * FOUND at array nr $i\n" if $#new >= 0;
            }
        }
    }
    return @array;
}

use XML::Simple;        # from package libxml-simple-perl
sub Get_weather_forecast { # get weather forecast data usually next day 48 hours
    state %Wcast;
    my $location = shift; my %weather; my $data; my @arr;
    my $json = JSON->new->allow_nonref;
    $location = $locations{$location_dflt}{weather} if not defined $location;
    $location = $locations{$location}{weather} if defined $locations{$location}{weather};
    return $Wcast{$location} if defined $Wcast{$location};

    $data = get_cache('weather');
    if( not defined $data ){
        # meteo forecast via Norway weather service
        # Norwegian Meteorological Institute and the NRK
        my $url = "http://www.yr.no/place/$location/forecast_hour_by_hour.xml";

        # next did not work as json content should have @attribute elements
        # TO DO change the js script to use the json without @attributes
        # my $xml = get_file($url);
        # if( not defined $xml ){
        #     print STDERR "Forecast FAILURE: Cannot obtain weather forecast via $url\n";
        #     return undef;
        # }
        # 
        # # Create the object of XML Simple and convert xml to perl format
        # my $xmlSimple = new XML::Simple(KeepRoot   => 1);
        # $data = $xmlSimple->XMLin($xml);
        
        # we use php script to download the xml data, convert it to json
        # convert json to perl struct so it has @attribute keys
        my $IN; open $IN, "/usr/bin/php -r '
                \$xml = file_get_contents(\"$url\");
                \$xml = simplexml_load_string(\$xml);
                \$json = json_encode(\$xml);
                echo \"\$json\";
                ' 2>/dev/null |";
        if( (not defined $IN) || $? ) {
            print STDERR "Forecast FAILURE: Cannot obtain weather json forecast via $url\n";
            return undef;
        }
        local $/; 
        $data = <$IN>; close $IN;
        if( length($data) < 50 ) {
            print STDERR "Forecast FAILURE: Cannot obtain weather json forecast via $url. Connection erorr.\n";
            return undef;
        }
        $data = $json->decode($data);
        if( ($archive > 0) && $debug ){
            my $FILE;
            my $name = strftime("Weather-Forecast-%Y-%m-%d_%Hh%M.json",localtime(time));
            open $FILE, ">./$name";
            print STDERR "Forecast Created weather forecast $location file: ./$name\n"
                if $verbose;
            print $FILE $json->pretty->encode($data);
            close $FILE;
        }
        
        @arr = WalkTree(\$data,'meta');
        if( $#arr >= 0 ){
            ${$arr[0]}->{nextupdate} =~ s/T/ /;
            ${$arr[0]}->{lastupdate} =~ s/T/ /;
            print STDERR "Forecast Weather: last updated at ${$arr[0]}->{lastupdate}, next update at ${$arr[0]}->{nextupdate}\n" if $debug;
        }
        $cache{weather}{lastupdate} = time;
        $cache{weather}{nextupdate} = Time::Piece->strptime(${$arr[0]}->{nextupdate}." +0100","%Y-%m-%d %H:%M:%S %z")->epoch;
        $cache{weather}{nextupdate} += 90*60;
        $cache{weather}{data} = $data;
        $cache{flagged} = 1; # wait for trenslate to update cache in file system
        $data = \$cache{weather}{data};
    }
        
    # use encode json function to convert xml object in json.
    #$weather{forecast} = translate($json->encode(${$data}));
    $cache{flagged} = 1 if Translate_names($data,'name');
    store_cache();
    
    $weather{forecast} = $json->encode(${$data});
    #@arr = WalkTree(${$data},'time'); # time is an array of hourly values
    # if( ($#arr < 0 ) || (not defined ${$arr[0]}->[0]{from}) ){
    @arr = WalkTree(${$data},'from');
    if( ($#arr < 0 ) || (not defined $arr[0]) ) {
        print STDERR "Forecast FAILURE: Weather json file has no array with timed data!.\n";
        return undef;
    }
    my $first = 2*time;
    for( my $i = 0; $i < $#arr; $i++ ) {
        my $hr = Time::Piece->strptime(${$arr[$i]}." +0100","%Y-%m-%dT%H:%M:%S %z")->epoch;
        if( $first > $hr ) {
            $weather{start} = ${$arr[$i]}; $first = $hr;
        }
    }
    # $weather{start} = ${$arr[0]}->[0]{from}; # we think array is ordered by date
    $weather{start} =~ s/T/ /;
    $weather{hours} = $#arr+1;   # $#{$arr} + 1;
    printf STDERR ("Forecast Found weather first time: %s, with %d hourly values.\n",
        $weather{start}, $weather{hours}) if $verbose > 1;
    $weather{start} = sprintf("%d",Time::Piece->strptime($weather{start}." +0100","%Y-%m-%d %H:%M:%S %z")->epoch);
    if( $verbose ){
        # $weather{start} .= sprintf("\n// Weather info start %s, %s hours\n",
        #         strftime("%Y-%m-%d %H:%M",localtime(integer($weather{start}))),
        #         $weather{hours} < 0 ? '?' : $weather{hours}); 
        printf STDERR ("Forecast var weather = %s\n",$weather{forecast}) if $debug > 1;
    }
    $Wcast{$location} = \%weather;
    return \%weather;
}
# Get_weather_forecast('Grubbenvorst');

# generate from template file with Highcharts definitions the html chart
# arguments:    aqi type (optional dflt LKI),
#               output file (dflt stdout) optional,
#               location identifier, see %locations optional dflt $location_dflt
#               div identifier,  optional default FORECASTING
#               input chart template file (optional,
#                       if not defined DATA script part
sub Generate {
    # default AQI type (LKI|AQI), template filoe can define AQI type as well!
    my $aqi_type = shift; $aqi_type = 'LKI' if not defined $aqi_type;
    $aqi_type = 'LKI' if $aqi_type =~ /(both|all)/; # all is not yet supported
    # JS script output, with debug on HTML will be included
    my $output = shift; $output = '/dev/stdout' if not defined $output;
    if ( $output !~ /^\/dev/ ) {
        $output =~ s/\.html//i;
        $output .= '.html' if $debug;
    }
    # default location as defined in $locations for URL download access
    my $location = shift; $location = $location_dflt if not defined $location;
    # default  id
    #   <div id="FORECASTING" style="width: 800px; height: 310px; margin: 0 auto">
    #     <div style="margin-top: 100px; text-align: center" id="loading">
    #       <i class="fa fa-spinner fa-spin"></i> Loading data from external source
    #     </div>
    #   </div>
    my $identifier = shift; $identifier = 'FORECASTING' if not defined $identifier;
    # HTML JS script template file
    my $template = shift; $template = $Dflt_template
        if (not defined $template) && (defined $Dflt_template);

    if ( not defined $locations{$location} ){
        print STDERR "Forecast ERROR: $location location URL info details are not defined!";
        return 0;
    }
    $cache_file =~ s/X+/$location/;      # per location a cache file
    if( $cache_file =~ /^(.*)\/[^\/]+/ ) {
        my $dir = $1; mkdir($dir,0770) if not -d $dir;
    }
    
    if ( (defined $template) && (not -r $template) ) {
        printf STDERR "Forecast ERROR: file $template does not exist or is nor readable.\n";
        return 0;
    }
    my $IN; my $OUT; my $GLOB; my $DOM; my $DBG; my $RSLT= 'ERRORS encountered';
    if ( (defined $template) && $template ) {
        open $IN, "<$template" || die ("Cannot open template file $template\n");
    }
    if( $debug ) {
        open $DBG, ">$output" || die ("Cannot open $output for generated HTML code\n");
        $RSLT="HTML page ready output on $output.\n";
        $OUT = $DBG; $GLOB = $DBG; $DOM = $DBG;
    } else {
        $output .= '.GLOB.json' if $output !~ /^\/dev\//;
        open $GLOB, ">$output" || die ("Cannot open ${output} for generated HTML code\n");
        $RSLT="File=$output with GLOBAL JS script.\n";
        $output =~ s/GLOB.json$/DOM.json/ if $output !~ /^\/dev\//;
        open $DOM, ">$output" || die ("Cannot open ${output} for generated HTML code\n");
        $RSLT .= "File=$output with DOM function JS script.\n";
        open $DBG, ">/dev/null" || die ("Cannot open /dev/null for generated HTML garbage\n");
        $OUT = $DBG;
    }
    # download weather info and convert it to json ready for Hightcharts JS scripts
    my $weather = Get_weather_forecast($location);
    if( not defined $weather ){
        print STDERR "Forecast ERROR: failed to obtain weather forecasts for $location\n";
        return 0
    }
    # use the downloaded data to generate HTML/JS script code from the template file
    my $inscript = 0; my $skip = '';
    my $PM25 = json_PM_forecast($location,$aqi_type,$weather->{start},$weather->{hours});
    if( (not defined $PM25) || (not defined $PM25->{$aqi_type}) ){
        print STDERR "Forecast ERROR: failed to obtain particular matter forecast for $aqi_type values, location $location\n";
        return 0;
    }
    my $inheader = 0; my $line_cnt = 0;
    while( 1 ){            # parse template file
        if ( $IN ) { $_ = <$IN> ; }
        else { $_ = <main::DATA> }
        last if not $_; $line_cnt++;
        print STDERR $_ if $debug > 2;
        if( /<head>/ ){ $inheader = 1; }
        if( /<\/head>/ ){ $inheader = 0; }
        if( (/<script\s/) && (not $inheader) ){ $inscript = 1; };
        if( /<\/script>/ ){ $inscript = 0; };
        next if (not $inscript) && (not $debug);
        # output switcher: debug, dom ready function part and script global part
        if( /^\/\/\s+FORECAST\s+start\s+(DOM|GLOB)/ ) {
            my $sw = $1;
            $OUT = $DOM if $sw =~ /DOM/;
            $OUT = $GLOB if $sw =~ /GLOB/;
        } elsif ( /^\/\/\s+FORECAST\s+end/ ) {
            print $OUT $_;
            $OUT = $DBG;
            next;
        } 
        if( /^\/\/\s+START\s+([a-zA-Z0-9]*).*/ ){ # insert new values
            $type = $1;
            print $OUT $_ if $debug; my $do_skip = 1;
            
            if( $type =~ /^(start)/ ){
                printf $OUT ("%se3;\n",$PM25->{start});
            } elsif( $type =~ /^LastUpdate/i ) {
                my $update = '';
                $update = $cache{'pollutant'}{lastupdate} if defined $cache{'pollutant'}{lastupdate};
                $update = $cache{'weather'}{lastupdate}
                    if (defined $cache{'weather'}{lastupdate}) && $update
                        && ($update < $cache{'weather'}{lastupdate});
                $update = strftime(" <div style=\"font-size:80%%\">(bijgewerkt op %a %d %b %k:%M)</dev>",localtime($update))
                    if $update;
                printf $OUT ("'%s';",$update);
            } elsif( $type =~ /^(pm25|colors|qual[a-z]*)\s*(LKI|AQI)?/ ){
                # insert PM aqi/color/qualifier info as json
                my $elmnt = $1; my $aqi = $2;
                if( not $aqi ){ $aqi = $aqi_type; }
                if( not defined $PM25->{$aqi} ){
                    $PM25 = json_PM_forecast($locations{$location}{PM},$aqi,$weather->{start},$weather->{hours});
                    if( not defined $PM25->{$aqi}{$elmnt} ){
                        print STDERR "Forecast ERROR: failed to obtain particular matter forecast for $aqi ($elmnt) values, location $location\n";
                    }
                }
                printf $OUT ("%s;\n",$PM25->{$aqi}{$elmnt}) if defined $PM25->{$aqi};
            } elsif( $type =~ /^(wstart)/ ){ # insert start time weather info
                # needed to synchronize pollutant and weather data
                printf $OUT ("%se3;\n",$weather->{start});
            } elsif( $type =~ /^(weather|forecast)/ ){
                # insert weather forecast info as json
                my $elmnt = $1; $elmnt =~ s/weather/forecast/;
                printf $OUT ("%s;\n", $weather->{$elmnt});
            } elsif( $type =~ /aqi/ ){ # insert aqi Index type: LKI or AQI
                printf $OUT ("['%s',];\n", $aqi_type) if $aqi_type !~ /all/;
                printf $OUT ("['LKI','AQI'];\n", $aqi_type) if $aqi_type =~ /all/;
            } elsif( $type =~ /symbols/ ) { # weather symbol collection
                $symbolsURL =~ s%http://behouddeparel.nl%file:///webdata/Drupal/sites% if $debug;
                printf $OUT ("'%s';\n", $symbolsURL);
            } else { $do_skip = 0; }
            if( $do_skip ){
              while(1){
                if ( $IN ) { $_ = <$IN> ; }
                else { $_ = <main::DATA> }
                last if not $_;
                print STDERR $_ if $debug > 2;
                if( /^\/\/\s+END/ ){
                    $type = '';
                    last;
                }
              }
            }
        }
        if ( /FORECASTING/ ) {     # div identifier
            s/id="FORECASTING"/id='${identifier}_${aqi_type}'/ if $aqi_type !~ /all/;
            s/FORECASTING/$identifier/g;
        }
        s/\/\/\s.*// if not $debug;
        next if /^\s*$/;
        print $OUT $_;
    }
    close $IN if $IN;
    print STDERR $RSLT if $verbose > 1;
    if( $line_cnt < 1 ) {
       print STDERR "Empty forecast file, location $location\n"; return 0;
    }
    return 1;
}
    
sub Usage {
    print "
Command:
forecast.pl [options] aqi_type output_file location [identifier] [template_file]
options: [-h|-d|-q|-v|-a|-w dir]

Generates from template file with Highcharts definitions
The forecast data is downloaded from yr.no (weather) and aqicn.org (PM2.5).
the html chart can be identified by the identifier (<div id='FORECASTING_AqiType'>...</div>).

Arguments:    aqi type          LKI or AQI or all (dflt LKI),
                                Type 'all' is not yet supported (degradated to LKI).
              output file name  (dflt stdout) (at debug HTML else JS script),
                                In debug mode extent name with .html
                                Otherwise 2 file are created:
                                with .DOM.json extension as DOM ready part
                                and .GLOB.json extension as global definitions part.
              location identifier, see %locations (dflt $location_dflt),
              div identifier (dflt FORECASTING extended with _aqi_type)
                                div identifier will be extended with '_aqi_type' .
              template input    HTML chart template file (dflt from DATA
                                embedded in the perl script),
Options:      --help|-h usage/help message
              --debug|-d increase debug level (level 1: output full HTML file)
                (debug: output is full HTML page on stdout).
              --quiet|-q quiet
              --verbose|-v increase verbose level (dflt 1)
                (level 2 output json arrays)
              --archive|-a archive downloaded json data in current directory
              --working|-w 'dir'  working directory,
                dir with all data files (dflt './')

Installation:
    Make sure to install ./forecast.html template file in working dir.
    Add the chart lib files to force page load with these chart lib files.
    Chart needs the following files
    (install them in Drupal .../sites/all/libraries/highcharts/
        https://ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js
        https://code.highcharts.com/highcharts.js
        https://code.highcharts.com/modules/exporting.js
        In header as link:
        https://netdna.bootstrapcdn.com/font-awesome/4.0.3/css/font-awesome.css
        In some image direcory (see: $symbolsURL):
        https://www.highcharts.com/samples/graphics/meteogram-symbols-30px.png
";
    exit 0;
}

while ( (defined $ARGV[0]) && ($ARGV[0] =~ /^-/) ){
    if( $ARGV[0] =~ /^--?h/ ){ Usage(); }
    if( $ARGV[0] =~ /^--?d/ ){ $debug++ ; }
    if( $ARGV[0] =~ /^--?v/ ){ $verbose++; }
    if( $ARGV[0] =~ /^--?q/ ){ $verbose = 0; }
    if( $ARGV[0] =~ /^--?a/ ){ $archive++; }
    if( $ARGV[0] =~ /^--?w/ ){ 
        if( -d $ARGV[1] ){ chdir $ARGV[1]; shift ; shift; next }
        else { print "ERROR: $ARGV[1] dir not found!\n"; exit 1; }
    }
    shift
}
$verbose += $debug;
$archive += $debug;
if( Generate(@ARGV) ){ exit 0; }
exit 1;

__DATA__
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
<script src='https://ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js'></script>
<script src="https://code.highcharts.com/highcharts.js"></script>
<script src="https://code.highcharts.com/modules/exporting.js"></script>
<!--
<link href="https://netdna.bootstrapcdn.com/font-awesome/4.0.3/css/font-awesome.css" rel="stylesheet">
-->
</head>
<body>
<script type="text/javascript">
        Highcharts.setOptions({
            lang: {
                months: ['januari','februari','maart','april','mei','juni','juli','augustus','september','october','november','december'],
                shortMonths: ['jan','feb','mrt','apr','mei','jun','jul','aug','sep','oct','nov','dec'],
                weekdays: ['zondag','maandag','dinsdag','woensdag','donderdag','vrijdag','zaterdag'],
                shortWeekdays: ['zo','ma','di','wo','do','vr','za'],
        
rangeSelectorTo: 'tot',
rangeSelectorFrom: 'van',

 }
 });
// FORECAST start GLOB
var LastUpdate =
// START LastUpdate
'';
// END LastUpdate
var fijnstofStart =
// START start
1480417200e3;
// END start
var fijnstofAQI =
// START pm25
[2.7,2.7,3,3,3,3,3,3,3,3,3,3,3,3.1,3.2,3.4,3.5,3.5,3.5,3.7,4,4,4.1,4.1,3.6,3.9,4,3.2,3.5,3.6,2.7,3,3.1,2.7,2.7,3,2.7,3,3,3,3,3.1,2.5,2.7,3,2,2.2,2.2,2.4];
// END pm25
var fijnstofColors =
// START colors
["#C4ECFD","#C4ECFD","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFDA4","#FFFDA4","#FFFDA4","#FFFDA4","#FFFDA4","#FFFDA4","#FFFDA4","#FFFDA4","#FFFED0","#FFFED0","#FFFDA4","#C4ECFD","#FFFED0","#FFFED0","#C4ECFD","#C4ECFD","#FFFED0","#C4ECFD","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#FFFED0","#C4ECFD","#C4ECFD","#FFFED0","#2DCDFB","#2DCDFB","#2DCDFB","#2DCDFB"];
// END colors
var fijnstofQualifications =
// START qual
["goed","goed","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","matig","goed","matig","matig","goed","goed","matig","goed","matig","matig","matig","matig","matig","goed","goed","matig","goed","goed","goed","goed"];
// END qual
var AQIlevels = {LKI:3,AQI:50};
var AQItype =
// START aqi
['LKI',];
// END aqi

// need weather symbol collection to select a symbol from
var symbolsURL =
// START symbols
'https://www.highcharts.com/samples/graphics/meteogram-symbols-30px.png';
// END symbols

/**
 * This is a complex demo of how to set up a Highcharts chart, coupled to a
 * dynamic source and extended by drawing image sprites, wind arrow paths
 * and a second grid on top of the chart. The purpose of the demo is to inpire
 * developers to go beyond the basic chart types and show how the library can
 * be extended programmatically. This is what the demo does:
 *
 * - Loads weather forecast from www.yr.no in form of an XML service. The XML
 *   is translated on the Higcharts website into JSONP for the sake of the demo
 *   being shown on both our website and JSFiddle.
 * - When the data arrives async, a Meteogram instance is created. We have
 *   created the Meteogram prototype to provide an organized structure of the different
 *   methods and subroutines associated with the demo.
 * - The parseYrData method parses the data from www.yr.no into several parallel arrays. These
 *   arrays are used directly as the data option for temperature, precipitation
 *   and air pressure. As the temperature data gives only full degrees, we apply
 *   some smoothing on the graph, but keep the original data in the tooltip.
 * - After this, the options structure is build, and the chart generated with the
 *   parsed data.
 * - In the callback (on chart load), we weather icons on top of the temperature series.
 *   The icons are sprites from a single PNG image, placed inside a clipped 30x30
 *   SVG <g> element. VML interprets this as HTML images inside a clipped div.
 * - Lastly, the wind arrows are built and added below the plot area, and a grid is
 *   drawn around them. The wind arrows are basically drawn noord-zuid, then rotated
 *   as per the wind direction.
 */

function Meteogram(xml, container) {
    // Parallel arrays for the chart data, these are populated as the XML/JSON file
    // is loaded
    this.symbols = [];
    this.symbolNames = [];
    this.precipitations = [];
    this.windDirections = [];
    this.windDirectionNames = [];
    this.windSpeeds = [];
    this.windSpeedNames = [];
    this.temperatures = [];
    this.pressures = [];

    // Initialize
    this.xml = xml;
    this.container = container;

    // Run
    this.parseYrData();
}
/**
 * Return weather symbol sprites as laid out at http://om.yr.no/forklaring/symbol/
 */
Meteogram.prototype.getSymbolSprites = function (symbolSize) {
    return {
        '01d': {
            x: 0,
            y: 0
        },
        '01n': {
            x: symbolSize,
            y: 0
        },
        '16': {
            x: 2 * symbolSize,
            y: 0
        },
        '02d': {
            x: 0,
            y: symbolSize
        },
        '02n': {
            x: symbolSize,
            y: symbolSize
        },
        '03d': {
            x: 0,
            y: 2 * symbolSize
        },
        '03n': {
            x: symbolSize,
            y: 2 * symbolSize
        },
        '17': {
            x: 2 * symbolSize,
            y: 2 * symbolSize
        },
        '04': {
            x: 0,
            y: 3 * symbolSize
        },
        '05d': {
            x: 0,
            y: 4 * symbolSize
        },
        '05n': {
            x: symbolSize,
            y: 4 * symbolSize
        },
        '18': {
            x: 2 * symbolSize,
            y: 4 * symbolSize
        },
        '06d': {
            x: 0,
            y: 5 * symbolSize
        },
        '06n': {
            x: symbolSize,
            y: 5 * symbolSize
        },
        '07d': {
            x: 0,
            y: 6 * symbolSize
        },
        '07n': {
            x: symbolSize,
            y: 6 * symbolSize
        },
        '08d': {
            x: 0,
            y: 7 * symbolSize
        },
        '08n': {
            x: symbolSize,
            y: 7 * symbolSize
        },
        '19': {
            x: 2 * symbolSize,
            y: 7 * symbolSize
        },
        '09': {
            x: 0,
            y: 8 * symbolSize
        },
        '10': {
            x: 0,
            y: 9 * symbolSize
        },
        '11': {
            x: 0,
            y: 10 * symbolSize
        },
        '12': {
            x: 0,
            y: 11 * symbolSize
        },
        '13': {
            x: 0,
            y: 12 * symbolSize
        },
        '14': {
            x: 0,
            y: 13 * symbolSize
        },
        '15': {
            x: 0,
            y: 14 * symbolSize
        },
        '20d': {
            x: 0,
            y: 15 * symbolSize
        },
        '20n': {
            x: symbolSize,
            y: 15 * symbolSize
        },
        '20m': {
            x: 2 * symbolSize,
            y: 15 * symbolSize
        },
        '21d': {
            x: 0,
            y: 16 * symbolSize
        },
        '21n': {
            x: symbolSize,
            y: 16 * symbolSize
        },
        '21m': {
            x: 2 * symbolSize,
            y: 16 * symbolSize
        },
        '22': {
            x: 0,
            y: 17 * symbolSize
        },
        '23': {
            x: 0,
            y: 18 * symbolSize
        },
        '46': {
            x: symbolSize,
            y: 8 * symbolSize
        }
    };
};


/**
 * Function to smooth the temperature line. The original data provides only whole degrees,
 * which makes the line graph look jagged. So we apply a running mean on it, but preserve
 * the unaltered value in the tooltip.
 */
Meteogram.prototype.smoothLine = function (data) {
    var i = data.length,
        sum,
        value;

    while (i--) {
        data[i].value = value = data[i].y; // preserve value for tooltip

        // Set the smoothed value to the average of the closest points, but don't allow
        // it to differ more than 0.5 degrees from the given value
        sum = (data[i - 1] || data[i]).y + value + (data[i + 1] || data[i]).y;
        data[i].y = Math.max(value - 0.5, Math.min(sum / 3, value + 0.5));
    }
};

/**
 * Callback function that is called from Highcharts on hovering each point and returns
 * HTML for the tooltip.
 */
Meteogram.prototype.tooltipFormatter = function (tooltip) {

    // Create the header with reference to the time interval
    var index = tooltip.points[0].point.index,
        ret = '<small>' + Highcharts.dateFormat('%A, %b %e, %H:%M', tooltip.x) + '-' +
            Highcharts.dateFormat('%H:%M', tooltip.points[0].point.to) + '</small><br>';

    // Symbol text
    ret += '<b>' + this.symbolNames[index] + '</b>';

    ret += '<table>';

    // Add all series
    Highcharts.each(tooltip.points, function (point) {
        var series = point.series;
        ret += '<tr><td><span style="color:' + series.color + '">\u25CF</span> ' + series.name +
            ': </td><td style="white-space:nowrap">' + Highcharts.pick(point.point.value, point.y) +
            series.options.tooltip.valueSuffix +
            (series.options.tooltip.valueSuffix.match(/Index/) ? ' <span style="font-size:120%;font-weight:bold;color:'+fijnstofColors[index]+';text-shadow:1px 1px 1px #000000">' + fijnstofQualifications[index] + '</span>' : '') +
            '</td></tr>';
    });

    // Add wind
    ret += '<tr><td style="vertical-align: top">\u25CF wind</td><td style="white-space:nowrap">' + this.windDirectionNames[index] +
        '<br>' + this.windSpeedNames[index] + ' (' +
        Highcharts.numberFormat(this.windSpeeds[index], 1) + ' m/s)</td></tr>';

    // Close
    ret += '</table>';


    return ret;
};

/**
 * Draw the weather symbols on top of the temperature series. The symbols are sprites of a single
 * file, defined in the getSymbolSprites function above.
 */
Meteogram.prototype.drawWeatherSymbols = function (chart) {
    var meteogram = this,
        symbolSprites = this.getSymbolSprites(18);

    $.each(chart.series[0].data, function (i, point) {
        var sprite,
            group;

        if (meteogram.resolution > 36e5 || i % 2 === 0) {

            sprite = symbolSprites[meteogram.symbols[i]];
            if (sprite) {

                // Create a group element that is positioned and clipped at 18 pixels width and height
                group = chart.renderer.g()
                    .attr({
//                        translateX: point.plotX + chart.plotLeft - 18/2 -4 // -7,
                        translateX: point.plotX + chart.plotLeft -4, //-7,
 //                       translateY: point.plotY + chart.plotTop - 18,
                        translateY: chart.plotTop,
                        zIndex: 5
                    })
                    .clip(chart.renderer.clipRect(0, 0, 18, 18))
                    .add();

                // Position the image inside it at the sprite position
                chart.renderer.image(
                    symbolsURL,
                    -sprite.x,
                    -sprite.y,
                    3*18,
                    19*18
                )
                    .add(group);
            }
        }
    });
};

/**
 * Create wind speed symbols for the Beaufort wind scale. The symbols are rotated
 * around the zero centerpoint.
 */
Meteogram.prototype.windArrow = function (name) {
    var level,
        path;

    // The stem and the arrow head
    path = [
        'M', 0, 4, //7, // base of arrow
        'L', -1.5, 4, //7,
        0, 6, //10,
        1.5, 4, //7,
        0, 4, //7,
        0, -6, //-10 // top
    ];

    level = $.inArray(name,
//    ['Calm','Clear air','Light breeze','Gentle breeze','Moderate breeze','Fresh breeze','Strong breeze','Near gale','Gale','Strong gale','Storm','Violent storm','Hurricane']
    ['windstilte','heldere lucht','lichte wind','matige bries','frisse bries','bries','harde wind','krachtige wind','stormachtig','storm','zware storm','orkaan']
    );

    if (level === 0) {
        path = [];
    }
    level = 1; // no speed indicators

    if (level === 2) {
        path.push('M', 0, -8, 'L', 4, -8); // short line
    } else if (level >= 3) {
        path.push(0, -10, 7, -10); // long line
    }

    if (level === 4) {
        path.push('M', 0, -7, 'L', 4, -7);
    } else if (level >= 5) {
        path.push('M', 0, -7, 'L', 7, -7);
    }

    if (level === 5) {
        path.push('M', 0, -4, 'L', 4, -4);
    } else if (level >= 6) {
        path.push('M', 0, -4, 'L', 7, -4);
    }

    if (level === 7) {
        path.push('M', 0, -1, 'L', 4, -1);
    } else if (level >= 8) {
        path.push('M', 0, -1, 'L', 7, -1);
    }

    return path;
};

/**
 * Draw the wind arrows. Each arrow path is generated by the windArrow function above.
 */
Meteogram.prototype.drawWindArrows = function (chart) {
    var meteogram = this;

    $.each(chart.series[0].data, function (i, point) {
        var arrow, x, y;

        if (meteogram.resolution > 36e5 || i % 2 === 0) {

            // Draw the wind arrows
            x = point.plotX + chart.plotLeft + 3; //7;
            y = 255-8;
            if (meteogram.windSpeedNames[i] === 'windstilte') {
                arrow = chart.renderer.circle(x, y, 7 /*10*/).attr({
                    fill: 'none'
                });
            } else {
                arrow = chart.renderer.path(
                    meteogram.windArrow(meteogram.windSpeedNames[i])
                ).attr({
                    rotation: parseInt(meteogram.windDirections[i], 10),
                    translateX: x, // rotation center
                    translateY: y // rotation center
                });
            }
            arrow.attr({
                stroke: (Highcharts.theme && Highcharts.theme.contrastTextColor) || 'black',
                'stroke-width': 1.5,
                zIndex: 5
            })
            .add();

        }
    });
};

/**
 * Draw blocks around wind arrows, below the plot area
 */
Meteogram.prototype.drawBlocksForWindArrows = function (chart) {
    var xAxis = chart.xAxis[0],
        x,
        pos,
        max,
        isLong,
        isLast,
        i;

    for (pos = xAxis.min, max = xAxis.max, i = 0; pos <= max + 36e5; pos += 36e5, i += 1) {

        // Get the X position
        isLast = pos === max + 36e5;
        x = Math.round(xAxis.toPixels(pos)) + (isLast ? 0.5 : -0.5);

        // Draw the vertical dividers and ticks
        if (this.resolution > 36e5) {
            isLong = pos % this.resolution === 0;
        } else {
            isLong = i % 2 === 0;
        }
        chart.renderer.path(['M', x, chart.plotTop + chart.plotHeight + (isLong ? 0 : 18 /*28*/),
            'L', x, chart.plotTop + chart.plotHeight + (isLong ? 26 : 24) /*32*/, 'Z'])
            .attr({
                'stroke': chart.options.chart.plotBorderColor,
                'stroke-width': 1
            })
            .add();
    }
};

/**
 * Get the title based on the XML data
 */
Meteogram.prototype.getTitle = function () {
    return 'verwachting lucht kwaliteit '+AQItype[0]+' (PM<span style="font-size:70%">2.5</span>) Index en weer';
};

/**
 * Build and return the Highcharts options structure
 */
Meteogram.prototype.getChartOptions = function () {
    var meteogram = this;

    return {
        chart: {
            renderTo: this.container,
            marginBottom: 75, //70,
            // marginRight: 40,
            marginTop: 55, //50,
            plotBorderWidth: 1,
            // width: 800,
            width: 540, // force width
            height: 310,
            spacingRight: 15, //40,
            spacingLeft: 0,
            spacingBottom: 15,
            borderRadius: 3,
            borderWidth: 1,
            backgroundColor: '#ffffff',
            borderColor: 'silver',
            shadow: true,
        },
        subtitle: {
            text: 'lokatie: ' + this.xml.location.name + LastUpdate,
            style: { fontSize: '12px', color: '#2A3B55' },
            align: 'center',
            y: 20
        },
        title: {
            text: this.getTitle(),
            style: { fontSize: '12px', color: '#2A3B55' },
            align: 'center'
        },
        credits: {
            text: 'weersverwachting: YR.no, fijnstof: AQICN.org en ESA.int (CAMS), chart techniek: highcharts.com',
            href: this.xml.credit.link['@attributes'].url,
            position: { x: -40, y: -10 }
        },
        tooltip: {
            shared: true,
            useHTML: true,
            formatter: function () {
                return meteogram.tooltipFormatter(this);
            }
        },
        xAxis: [{ // Bottom X axis
            type: 'datetime',
            tickInterval: 2 * 36e5, // two hours
            minorTickInterval: 36e5, // one hour
            tickLength: 0,
            gridLineWidth: 1,
            gridLineColor: (Highcharts.theme && Highcharts.theme.background2) || '#F0F0F0',
            startOnTick: false,
            endOnTick: false,
            minPadding: 0,
            maxPadding: 0,
            offset: 30-10,
            showLastLabel: true,
            labels: {
                format: '{value:%H}'
            }
        }, { // Top X axis
            linkedTo: 0,
            type: 'datetime',
            tickInterval: 24 * 3600 * 1000,
            labels: {
                format: '{value:<span style="font-size: 12px; font-weight: bold">%a</span> %b %e}',
                align: 'left',
                x: 3,
                y: -5
            },
            opposite: true,
            tickLength: 20,
            gridLineWidth: 1
        }],
        yAxis: [{ // temperature axis
            title: {
                text: "\u00B0C",
                offset: 0,
                align: 'high',
                rotation: 0,
                style: {
                    fontSize: '10px',
                },
                textAlign: 'left',
                x: -20,
            },
            labels: {
                format: "{value}\u00BA",
                style: {
                    fontSize: '10px'
                },
                x: -3
            },
            showLastLable: false,
            plotLines: [{ // zero plane
                value: 0,
                color: '#BBBBBB',
                width: 1,
                zIndex: 2
            }],
            // Custom positioner to provide even temperature ticks from top down
            tickPositioner: function () {
                var max = Math.ceil(this.max) + 1,
                    pos = max - 12, // start
                    ret;
                if (pos < this.min) {
                    ret = [];
                    while (pos <= max) {
                        ret.push(pos += 1);
                    }
                } // else return undefined and go auto
                return ret;
            },
            showLastLabel: false,
            maxPadding: 0.3,
            tickInterval: 1,
            gridLineColor: (Highcharts.theme && Highcharts.theme.background2) || '#F0F0F0'
        }, { // precipitation axis
            title: {
                text: 'mm',
                align: 'high',
                rotation: 0,
                style: {
                    fontSize: '10px',
                    color: '#0A3477',
                },
                x: 10,
            },
            labels: {
                // enabled: false
                style: {
                    fontSize: '10px',
                    color: '#0A3477',
                },
                x: -4,
            },
            gridLineWidth: 0,
            tickInterval: 1,
            tickLength: 1,
            showLastLabel: false,
            floor: 0
        }, { // Air pressure
            allowDecimals: false,
            title: { // Title on top of axis
                text: 'hPa',
                offset: 0,
                align: 'high',
                rotation: 0,
                style: {
                    fontSize: '10px',
//                    color: Highcharts.getOptions().colors[2]
                    color: '#0473b3'
                },
                textAlign: 'left',
                x: 3
            },
            labels: {
                style: {
                    fontSize: '8px',
//                    color: Highcharts.getOptions().colors[2]
                    color: '#0473b3'
                },
                y: 2,
                x: 3
            },
            gridLineWidth: 0,
            opposite: true,
            showLastLabel: false,
            floor: 0
        }, { // air Quality Index
            allowDecimals: false,
            title: { // Title on top of axis
                text: AQItype[0],
                offset: 0,
                align: 'high',
                rotation: 0,
                style: {
                    fontSize: '10px',
                    color: '#90007a'
                },
                textAlign: 'left',
                x: 5
            },
            labels: {
                style: {
                    fontSize: '10px',
                    color: '#90007a'
                },
                y: 2,
                x: 5
            },
            maxPadding: 0.118,
            gridLineWidth: 0,
            opposite: true,
            showLastLabel: false,
            floor: 0
        }],

        legend: {
            // title: { text: 'legendum' },
            layout: 'vertical',
            borderRadius: '3px',
            backgroundColor: 'rgba(196,206,228,0.5)',
            align: 'left',
            y: 65,
            x: 58,
            itemStyle: { fontSize: '75%', color: '#314877' },
            itemHiddenStyle: { color: '#4a6396' },
            verticalAlign: 'center',
            enabled: true
        },

        plotOptions: {
            series: {
                pointPlacement: 'between'
            },
            column: { shadow: true, colorByPoint: true, showInLegend: false },
        },
        series: [
        {
            name: 'PM<span style="font-size:60%">2.5</span>',
            data: this.aqi,
            type: 'column',
            tooltip: {
                valueSuffix: ' '+AQItype[0]+' Index'
            },
            dataLabels: {
                enabled: true,
                formatter: function () {
                    if (this.y >= (AQItype[0] === 'AQI' ? AQIlevels.AQI : AQIlevels.LKI) ) {
                        if( this.y < 10 ) { return Math.round(this.y*10+0.5)/10; }
                        else { return Math.round(this.y); }
                    }
                },
                style: { fontSize: '8px' }
            },
            yAxis: 3,
            pointPlacement: 'between',
            pointWidth: 10,
            pointPadding: 0
        },
        {
            name: 'temperatuur',
            data: this.temperatures,
            type: 'spline',
            marker: {
                enabled: false,
                states: {
                    hover: {
                        enabled: true
                    }
                }
            },
            tooltip: {
                valueSuffix: '&deg;C'
            },
            zIndex: 1,
            color: '#FF3333',
            threshold: 4,
            negativeColor: '#48AFE8',
        }, {
            name: 'neerslag',
            data: this.precipitations,
            // type: 'column',
            type: 'spline',
            color: '#0A3477',
            // color: '#68CFE8',
            yAxis: 1,
            groupPadding: 0,
            pointPadding: 0,
            borderWidth: 0,
            shadow: false,
            dataLabels: {
                enabled: true,
                formatter: function () {
                    if (this.y > 0.5) {
                        if( this.y < 10 ) { return Math.round(this.y*10+0.5)/10; }
                        else { return Math.round(this.y); }
                    }
                },
                style: {
                    fontSize: '8px'
                }
            },
            threshold: 0.01,
            negativeColor: (Highcharts.theme && Highcharts.theme.background2) || '#F0F0F0',
            dashStyle: 'Dash',
            marker: { enabled: false },
            tooltip: {
                valueSuffix: 'mm'
            }
        }, {
            name: 'luchtdruk',
//            color: Highcharts.getOptions().colors[2],
            color: '#0473b3',
            data: this.pressures,
            marker: {
                enabled: false
            },
            shadow: false,
            tooltip: {
                valueSuffix: ' hPa'
            },
            dashStyle: 'shortdot',
            yAxis: 2
        }]
    };
};

/**
 * Post-process the chart from the callback function, the second argument to Highcharts.Chart.
 */
Meteogram.prototype.onChartLoad = function (chart) {
    this.drawWeatherSymbols(chart);
    this.drawWindArrows(chart);
    this.drawBlocksForWindArrows(chart);
};

/**
 * Create the chart. This function is called async when the data file is loaded and parsed.
 */
Meteogram.prototype.createChart = function () {
    var meteogram = this;
    this.chart = new Highcharts.Chart(this.getChartOptions(), function (chart) {
        meteogram.onChartLoad(chart);
    });
};

/**
 * Handle the data. This part of the code is not Highcharts specific, but deals with yr.no's
 * specific data format
 */
Meteogram.prototype.parseYrData = function () {
    var meteogram = this,
        xml = this.xml,
        pointStart;
    if (!xml || !xml.forecast) {
        // $('#loading').html('<i class="fa fa-frown-o"></i> Failed loading data, please try again later');
        $('#loading').html('Failed loading data, please try again later');
        return;
    }
    meteogram.aqi = [];
    // The returned xml variable is a JavaScript representation of the provided XML,
    // generated on the server by running PHP simple_load_xml and converting it to
    // JavaScript by json_encode.
    $.each(xml.forecast.tabular.time, function (i, time) {
        // Get the times - only Safari can't parse ISO8601 so we need to do some replacements
        var indx = 0;
        var from = time['@attributes'].from + ' UTC',
            to = time['@attributes'].to + ' UTC';

        from = from.replace(/-/g, '/').replace('T', ' ');
        from = Date.parse(from);
        to = to.replace(/-/g, '/').replace('T', ' ');
        to = Date.parse(to);
        if (to > pointStart + 4 * 24 * 36e5) {
            return;
        }
        // If it is more than an hour between points, show all symbols
        if (i === 0) {
            meteogram.resolution = to - from;
        }
        // Populate the parallel arrays
        meteogram.symbols.push(time.symbol['@attributes']['var'].match(/[0-9]{2}[dnm]?/)[0]); // eslint-disable-line dot-notation
        meteogram.symbolNames.push(time.symbol['@attributes'].name);
        meteogram.temperatures.push({
            x: from,
            y: parseInt(time.temperature['@attributes'].value, 10),
            // custom options used in the tooltip formatter
            to: to,
            index: i
        });

        meteogram.precipitations.push({
            x: from,
            y: Math.abs(parseFloat(time.precipitation['@attributes'].value))
        });
        meteogram.windDirections.push(parseFloat(time.windDirection['@attributes'].deg));
        meteogram.windDirectionNames.push(time.windDirection['@attributes'].name);
        meteogram.windSpeeds.push(Math.abs(parseFloat(time.windSpeed['@attributes'].mps)));
        meteogram.windSpeedNames.push(time.windSpeed['@attributes'].name);

        meteogram.pressures.push({
            x: from,
            y: Math.abs(parseFloat(time.pressure['@attributes'].value))
        });

        for( indx = 0; indx <= fijnstofAQI.length; indx++ ) {
            /* interval is 3 hours, choose the first of 3 hours */
            if ( ((fijnstofStart + indx * 3600e3) >= from) && ((fijnstofStart + indx * 3600e3) < to) ) {
                if ( fijnstofAQI[indx-1] && fijnstofColors[indx-1] ) {
                    meteogram.aqi.push({
                        x: fijnstofStart + (indx-1) * 3 * 3600e3, /*from, */
                        y: fijnstofAQI[indx-1],
                        color: fijnstofColors[indx-1],
                        // name: fijnstofQualifications[indx-1]
                    });
                }
                /* break ; */
            }
        }
        if (i === 0) {
            pointStart = (from + to) / 2;
        }
    });

    // Smooth the spline
    this.smoothLine(this.temperatures);
    this.smoothLine(this.precipitations);

    // Create the chart when the data is loaded
    this.createChart();
};
// End of the Meteogram protype



// FORECAST end GLOB
$(function () { // On DOM ready...
// FORECAST start DOM
    var xml_start =
// START wstart
1480417200e3;
// END wstart
        while( fijnstofAQI.length > 0 ) {
            if( fijnstofStart >= xml_start ) { break ; }
            fijnstofAQI.shift(); fijnstofStart += 3600e3;
        }
    var xml = 
// START forecast
{"location":{"name":"Grubbenvorst","type":"Populated place","country":"Netherlands","timezone":{"@attributes":{"id":"Europe\/Amsterdam","utcoffsetMinutes":"60"}},"location":{"@attributes":{"altitude":"23","latitude":"51.42","longitude":"6.14583","geobase":"geonames","geobaseid":"2755051"}}},"credit":{"comment":[{},{}],"link":{"@attributes":{"text":"Weather forecast from Yr, delivered by the Norwegian Meteorological Institute and the NRK","url":"http:\/\/www.yr.no\/place\/Netherlands\/Limburg\/Grubbenvorst\/"}}},"links":{"link":[{"@attributes":{"id":"xmlSource","url":"http:\/\/www.yr.no\/place\/Netherlands\/Limburg\/Grubbenvorst\/forecast.xml"}},{"@attributes":{"id":"xmlSourceHourByHour","url":"http:\/\/www.yr.no\/place\/Netherlands\/Limburg\/Grubbenvorst\/forecast_hour_by_hour.xml"}},{"@attributes":{"id":"overview","url":"http:\/\/www.yr.no\/place\/Netherlands\/Limburg\/Grubbenvorst\/"}},{"@attributes":{"id":"hourByHour","url":"http:\/\/www.yr.no\/place\/Netherlands\/Limburg\/Grubbenvorst\/hour_by_hour"}},{"@attributes":{"id":"longTermForecast","url":"http:\/\/www.yr.no\/place\/Netherlands\/Limburg\/Grubbenvorst\/long"}}]},"meta":{"lastupdate":"2016-11-29T10:17:48","nextupdate":"2016-11-29T17:00:00"},"sun":{"@attributes":{"rise":"2016-11-29T08:16:01","set":"2016-11-29T16:31:20"}},"forecast":{"tabular":{"time":[{"@attributes":{"from":"2016-11-29T12:00:00","to":"2016-11-29T13:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"01d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"116.3","code":"ESE","name":"oost-zuidoost"}},"windSpeed":{"@attributes":{"mps":"2.0","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-2"}},"pressure":{"@attributes":{"unit":"hPa","value":"1036.0"}}},{"@attributes":{"from":"2016-11-29T13:00:00","to":"2016-11-29T14:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"01d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"126.9","code":"SE","name":"zuid-oost"}},"windSpeed":{"@attributes":{"mps":"2.2","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1035.6"}}},{"@attributes":{"from":"2016-11-29T14:00:00","to":"2016-11-29T15:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"01d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"124.4","code":"SE","name":"zuid-oost"}},"windSpeed":{"@attributes":{"mps":"2.1","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"1"}},"pressure":{"@attributes":{"unit":"hPa","value":"1035.1"}}},{"@attributes":{"from":"2016-11-29T15:00:00","to":"2016-11-29T16:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"01d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"114.9","code":"ESE","name":"oost-zuidoost"}},"windSpeed":{"@attributes":{"mps":"1.9","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"1"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.9"}}},{"@attributes":{"from":"2016-11-29T16:00:00","to":"2016-11-29T17:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"01d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"107.7","code":"ESE","name":"oost-zuidoost"}},"windSpeed":{"@attributes":{"mps":"1.5","name":"heldere lucht"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.8"}}},{"@attributes":{"from":"2016-11-29T17:00:00","to":"2016-11-29T18:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.99"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"99.6","code":"E","name":"oost"}},"windSpeed":{"@attributes":{"mps":"1.3","name":"heldere lucht"}},"temperature":{"@attributes":{"unit":"celsius","value":"-2"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.8"}}},{"@attributes":{"from":"2016-11-29T18:00:00","to":"2016-11-29T19:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.99"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"110.1","code":"ESE","name":"oost-zuidoost"}},"windSpeed":{"@attributes":{"mps":"1.4","name":"heldere lucht"}},"temperature":{"@attributes":{"unit":"celsius","value":"-3"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.9"}}},{"@attributes":{"from":"2016-11-29T19:00:00","to":"2016-11-29T20:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.99"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"123.1","code":"ESE","name":"oost-zuidoost"}},"windSpeed":{"@attributes":{"mps":"1.5","name":"heldere lucht"}},"temperature":{"@attributes":{"unit":"celsius","value":"-4"}},"pressure":{"@attributes":{"unit":"hPa","value":"1035.2"}}},{"@attributes":{"from":"2016-11-29T20:00:00","to":"2016-11-29T21:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.99"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"140.3","code":"SE","name":"zuid-oost"}},"windSpeed":{"@attributes":{"mps":"1.7","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-5"}},"pressure":{"@attributes":{"unit":"hPa","value":"1035.1"}}},{"@attributes":{"from":"2016-11-29T21:00:00","to":"2016-11-29T22:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.99"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"142.6","code":"SE","name":"zuid-oost"}},"windSpeed":{"@attributes":{"mps":"1.9","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-5"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.8"}}},{"@attributes":{"from":"2016-11-29T22:00:00","to":"2016-11-29T23:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.99"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"152.8","code":"SSE","name":"zuid-zuidoost"}},"windSpeed":{"@attributes":{"mps":"2.0","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-6"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.4"}}},{"@attributes":{"from":"2016-11-29T23:00:00","to":"2016-11-30T00:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.99"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"174.9","code":"S","name":"zuid"}},"windSpeed":{"@attributes":{"mps":"1.3","name":"heldere lucht"}},"temperature":{"@attributes":{"unit":"celsius","value":"-6"}},"pressure":{"@attributes":{"unit":"hPa","value":"1035.0"}}},{"@attributes":{"from":"2016-11-30T00:00:00","to":"2016-11-30T01:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"193.3","code":"SSW","name":"zuid-zuidwest"}},"windSpeed":{"@attributes":{"mps":"1.1","name":"heldere lucht"}},"temperature":{"@attributes":{"unit":"celsius","value":"-7"}},"pressure":{"@attributes":{"unit":"hPa","value":"1035.2"}}},{"@attributes":{"from":"2016-11-30T01:00:00","to":"2016-11-30T02:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"193.2","code":"SSW","name":"zuid-zuidwest"}},"windSpeed":{"@attributes":{"mps":"2.0","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-7"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.5"}}},{"@attributes":{"from":"2016-11-30T02:00:00","to":"2016-11-30T03:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"198.9","code":"SSW","name":"zuid-zuidwest"}},"windSpeed":{"@attributes":{"mps":"2.5","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-7"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.5"}}},{"@attributes":{"from":"2016-11-30T03:00:00","to":"2016-11-30T04:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"1","numberEx":"1","name":"heldere lucht","var":"mf\/01n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"221.5","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"2.0","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-7"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.9"}}},{"@attributes":{"from":"2016-11-30T04:00:00","to":"2016-11-30T05:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"mf\/03n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"210.3","code":"SSW","name":"zuid-zuidwest"}},"windSpeed":{"@attributes":{"mps":"2.8","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-7"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.5"}}},{"@attributes":{"from":"2016-11-30T05:00:00","to":"2016-11-30T06:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"mf\/03n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"214.3","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"3.6","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-7"}},"pressure":{"@attributes":{"unit":"hPa","value":"1033.6"}}},{"@attributes":{"from":"2016-11-30T06:00:00","to":"2016-11-30T07:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"mf\/03n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"230.9","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"3.1","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-7"}},"pressure":{"@attributes":{"unit":"hPa","value":"1033.9"}}},{"@attributes":{"from":"2016-11-30T07:00:00","to":"2016-11-30T08:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"mf\/03n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"228.4","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"3.3","name":"lichte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-7"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.3"}}},{"@attributes":{"from":"2016-11-30T08:00:00","to":"2016-11-30T09:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"mf\/03n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"222.3","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.0","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-7"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.4"}}},{"@attributes":{"from":"2016-11-30T09:00:00","to":"2016-11-30T10:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"03d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"223.9","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"3.8","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-6"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.4"}}},{"@attributes":{"from":"2016-11-30T10:00:00","to":"2016-11-30T11:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"2","numberEx":"2","name":"kalm","var":"02d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"221.7","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.4","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-5"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.1"}}},{"@attributes":{"from":"2016-11-30T11:00:00","to":"2016-11-30T12:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"2","numberEx":"2","name":"kalm","var":"02d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"232.0","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.6","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-2"}},"pressure":{"@attributes":{"unit":"hPa","value":"1034.2"}}},{"@attributes":{"from":"2016-11-30T12:00:00","to":"2016-11-30T13:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"2","numberEx":"2","name":"kalm","var":"02d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"234.1","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.9","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"-1"}},"pressure":{"@attributes":{"unit":"hPa","value":"1033.5"}}},{"@attributes":{"from":"2016-11-30T13:00:00","to":"2016-11-30T14:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"03d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"238.1","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.7","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"1"}},"pressure":{"@attributes":{"unit":"hPa","value":"1032.6"}}},{"@attributes":{"from":"2016-11-30T14:00:00","to":"2016-11-30T15:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"243.3","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.7","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"2"}},"pressure":{"@attributes":{"unit":"hPa","value":"1032.5"}}},{"@attributes":{"from":"2016-11-30T15:00:00","to":"2016-11-30T16:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"03d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"245.0","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.9","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"3"}},"pressure":{"@attributes":{"unit":"hPa","value":"1032.3"}}},{"@attributes":{"from":"2016-11-30T16:00:00","to":"2016-11-30T17:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"03d"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"241.3","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.3","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"2"}},"pressure":{"@attributes":{"unit":"hPa","value":"1031.8"}}},{"@attributes":{"from":"2016-11-30T17:00:00","to":"2016-11-30T18:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"mf\/03n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"240.6","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.2","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"1"}},"pressure":{"@attributes":{"unit":"hPa","value":"1031.5"}}},{"@attributes":{"from":"2016-11-30T18:00:00","to":"2016-11-30T19:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"mf\/03n.02"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"239.6","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.1","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1031.6"}}},{"@attributes":{"from":"2016-11-30T19:00:00","to":"2016-11-30T20:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"236.7","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.5","name":"milde bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1031.4"}}},{"@attributes":{"from":"2016-11-30T20:00:00","to":"2016-11-30T21:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"241.0","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.2","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1031.6"}}},{"@attributes":{"from":"2016-11-30T21:00:00","to":"2016-11-30T22:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"238.3","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.3","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1031.2"}}},{"@attributes":{"from":"2016-11-30T22:00:00","to":"2016-11-30T23:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"242.3","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.1","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1030.9"}}},{"@attributes":{"from":"2016-11-30T23:00:00","to":"2016-12-01T00:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"245.3","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.0","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1031.0"}}},{"@attributes":{"from":"2016-12-01T00:00:00","to":"2016-12-01T01:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"245.4","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.9","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1031.0"}}},{"@attributes":{"from":"2016-12-01T01:00:00","to":"2016-12-01T02:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"240.3","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.7","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"1"}},"pressure":{"@attributes":{"unit":"hPa","value":"1030.6"}}},{"@attributes":{"from":"2016-12-01T02:00:00","to":"2016-12-01T03:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"238.6","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.3","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"1"}},"pressure":{"@attributes":{"unit":"hPa","value":"1030.1"}}},{"@attributes":{"from":"2016-12-01T03:00:00","to":"2016-12-01T04:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"mf\/03n.06"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"234.5","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.7","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"1"}},"pressure":{"@attributes":{"unit":"hPa","value":"1029.7"}}},{"@attributes":{"from":"2016-12-01T04:00:00","to":"2016-12-01T05:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"2","numberEx":"2","name":"kalm","var":"mf\/02n.06"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"235.8","code":"SW","name":"zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.0","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"1"}},"pressure":{"@attributes":{"unit":"hPa","value":"1029.2"}}},{"@attributes":{"from":"2016-12-01T05:00:00","to":"2016-12-01T06:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"mf\/03n.06"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"237.4","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.8","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1028.8"}}},{"@attributes":{"from":"2016-12-01T06:00:00","to":"2016-12-01T07:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"2","numberEx":"2","name":"kalm","var":"mf\/02n.06"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"236.9","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.7","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1028.6"}}},{"@attributes":{"from":"2016-12-01T07:00:00","to":"2016-12-01T08:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"3","numberEx":"3","name":"gedeeltelijk bewolkt","var":"mf\/03n.06"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"237.5","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.0","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1028.3"}}},{"@attributes":{"from":"2016-12-01T08:00:00","to":"2016-12-01T09:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"243.7","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"4.6","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"0"}},"pressure":{"@attributes":{"unit":"hPa","value":"1028.2"}}},{"@attributes":{"from":"2016-12-01T09:00:00","to":"2016-12-01T10:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"245.1","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.1","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"1"}},"pressure":{"@attributes":{"unit":"hPa","value":"1028.4"}}},{"@attributes":{"from":"2016-12-01T10:00:00","to":"2016-12-01T11:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"242.4","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.4","name":"zachte bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"2"}},"pressure":{"@attributes":{"unit":"hPa","value":"1028.2"}}},{"@attributes":{"from":"2016-12-01T11:00:00","to":"2016-12-01T12:00:00"},"comment":[{},{}],"symbol":{"@attributes":{"number":"4","numberEx":"4","name":"bewolkt","var":"04"}},"precipitation":{"@attributes":{"value":"0"}},"windDirection":{"@attributes":{"deg":"240.9","code":"WSW","name":"west-zuidwest"}},"windSpeed":{"@attributes":{"mps":"5.8","name":"milde bries"}},"temperature":{"@attributes":{"unit":"celsius","value":"2"}},"pressure":{"@attributes":{"unit":"hPa","value":"1027.3"}}}]}}};
// END forecast

    // Set the hash to the yr.no URL we want to parse
//    if (!location.hash) {
//        var place = 'United_Kingdom/England/London';
//        //place = 'France/Rhône-Alpes/Val_d\'Isère~2971074';
//        //place = 'Norway/Sogn_og_Fjordane/Vik/Målset';
//        //place = 'United_States/California/San_Francisco';
//        //place = 'United_States/Minnesota/Minneapolis';
//        location.hash = 'https://www.yr.no/place/' + place + '/forecast_hour_by_hour.xml';
//
//    }

//    // Then get the XML file through Highcharts' jsonp provider, see
//    // https://github.com/highcharts/highcharts/blob/master/samples/data/jsonp.php
//    // for source code.
//    $.getJSON(
//        'https://www.highcharts.com/samples/data/jsonp.php?url=' + location.hash.substr(1) + '&callback=?',
//        function (xml) {
            window.meteogram = new Meteogram(xml, 'FORECASTING_'+AQItype[0]);
//        }
//    );

// FORECAST end DOM
});
</script>
<div id="FORECASTING" style="width: 100%; height: 310px; margin: 0 auto">
<!--
    <div style="margin-top: 100px; text-align: center" id="loading">
        <i class="fa fa-spinner fa-spin"></i> Loading data from external source
    </div>
-->
</div>
<!--
<div style="width: 800px; margin: 0 auto">
    <a href="#http://www.yr.no/place/United_Kingdom/England/London/forecast_hour_by_hour.xml">London</a>,
    <a href="#http://www.yr.no/place/France/Rhône-Alpes/Val_d\'Isère~2971074/forecast_hour_by_hour.xml">Val d'Isère</a>,
    <a href="#http://www.yr.no/place/United_States/California/San_Francisco/forecast_hour_by_hour.xml">San Francisco</a>,
    <a href="#http://www.yr.no/place/Norway/Vik/Vikafjell/forecast_hour_by_hour.xml">Vikjafjellet</a>
</div>
-->
        
</body>
</html>
